import { __extends, __decorate, __metadata } from 'tslib';
import { Injectable, NgModule } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { ApolloLink, Observable } from 'apollo-link';
import { BatchLink } from 'apollo-link-batch';
import { print } from 'graphql/language/printer';
import { prioritize, mergeHeaders, fetch } from 'apollo-angular-link-http-common';

var defaults = {
    batchInterval: 10,
    batchMax: 10,
    uri: 'graphql',
    method: 'POST',
};
var HttpBatchLinkHandler = /** @class */ (function (_super) {
    __extends(HttpBatchLinkHandler, _super);
    function HttpBatchLinkHandler(httpClient, options) {
        var _this = _super.call(this) || this;
        _this.httpClient = httpClient;
        _this.options = options;
        _this.batchInterval = options.batchInterval || defaults.batchInterval;
        _this.batchMax = options.batchMax || defaults.batchMax;
        var batchHandler = function (operations) {
            return new Observable(function (observer) {
                var body = _this.createBody(operations);
                var headers = _this.createHeaders(operations);
                var _a = _this.createOptions(operations), method = _a.method, uri = _a.uri, withCredentials = _a.withCredentials;
                if (typeof uri === 'function') {
                    throw new Error("Option 'uri' is a function, should be a string");
                }
                var req = {
                    method: method,
                    url: uri,
                    body: body,
                    options: {
                        withCredentials: withCredentials,
                        headers: headers,
                    },
                };
                var sub = fetch(req, _this.httpClient).subscribe({
                    next: function (result) { return observer.next(result.body); },
                    error: function (err) { return observer.error(err); },
                    complete: function () { return observer.complete(); },
                });
                return function () {
                    if (!sub.closed) {
                        sub.unsubscribe();
                    }
                };
            });
        };
        var batchKey = options.batchKey ||
            (function (operation) {
                return _this.createBatchKey(operation);
            });
        _this.batcher = new BatchLink({
            batchInterval: _this.batchInterval,
            batchMax: _this.batchMax,
            batchKey: batchKey,
            batchHandler: batchHandler,
        });
        return _this;
    }
    HttpBatchLinkHandler.prototype.createOptions = function (operations) {
        var context = operations[0].getContext();
        return {
            method: prioritize(context.method, this.options.method, defaults.method),
            uri: prioritize(context.uri, this.options.uri, defaults.uri),
            withCredentials: prioritize(context.withCredentials, this.options.withCredentials),
        };
    };
    HttpBatchLinkHandler.prototype.createBody = function (operations) {
        var _this = this;
        return operations.map(function (operation) {
            var includeExtensions = prioritize(operation.getContext().includeExtensions, _this.options.includeExtensions, false);
            var includeQuery = prioritize(operation.getContext().includeQuery, _this.options.includeQuery, true);
            var body = {
                operationName: operation.operationName,
                variables: operation.variables,
            };
            if (includeExtensions) {
                body.extensions = operation.extensions;
            }
            if (includeQuery) {
                body.query = print(operation.query);
            }
            return body;
        });
    };
    HttpBatchLinkHandler.prototype.createHeaders = function (operations) {
        return operations.reduce(function (headers, operation) {
            return mergeHeaders(headers, operation.getContext().headers);
        }, this.options.headers);
    };
    HttpBatchLinkHandler.prototype.createBatchKey = function (operation) {
        var context = operation.getContext();
        if (context.skipBatching) {
            return Math.random()
                .toString(36)
                .substr(2, 9);
        }
        var headers = context.headers &&
            context.headers.keys().map(function (k) { return context.headers.get(k); });
        var opts = JSON.stringify({
            includeQuery: context.includeQuery,
            includeExtensions: context.includeExtensions,
            headers: headers,
        });
        return prioritize(context.uri, this.options.uri) + opts;
    };
    HttpBatchLinkHandler.prototype.request = function (op) {
        return this.batcher.request(op);
    };
    return HttpBatchLinkHandler;
}(ApolloLink));
var HttpBatchLink = /** @class */ (function () {
    function HttpBatchLink(httpClient) {
        this.httpClient = httpClient;
    }
    HttpBatchLink.prototype.create = function (options) {
        return new HttpBatchLinkHandler(this.httpClient, options);
    };
    HttpBatchLink = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [HttpClient])
    ], HttpBatchLink);
    return HttpBatchLink;
}());

var PROVIDERS = [HttpBatchLink];
var HttpBatchLinkModule = /** @class */ (function () {
    function HttpBatchLinkModule() {
    }
    HttpBatchLinkModule = __decorate([
        NgModule({
            providers: PROVIDERS,
        })
    ], HttpBatchLinkModule);
    return HttpBatchLinkModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { HttpBatchLink, HttpBatchLinkHandler, HttpBatchLinkModule, PROVIDERS };
//# sourceMappingURL=ng.apolloLink.httpBatch.js.map
